@@ -24,42 +24,17 @@ import Exception (ghandle)
 type SymbolName = String
 type ModuleName = String
 
-findSymbol :: SymbolName -> [FilePath] -> GHC.Ghc [ModuleName]
-findSymbol symbol files = do
-   -- for the findsymbol command GHC shouldn't output any warnings
-   -- or errors to stdout for the loaded source files, we're only
-   -- interested in the module graph of the loaded targets
-   dynFlags <- GHC.getSessionDynFlags
-   _        <- GHC.setSessionDynFlags dynFlags { GHC.log_action = \_ _ _ _ _ ->
-#if __GLASGOW_HASKELL__ >= 800
-                                                 return . return $ () }
-#else
-                                                 return () }
-#endif
-
-   fileMods <- concat <$> mapM (findSymbolInFile symbol) files
-
-   -- reset the old log_action
-   _ <- GHC.setSessionDynFlags dynFlags
+findSymbol :: SymbolName -> GHC.Ghc [ModuleName]
+findSymbol symbol = do
+   fileMods <- findSymbolInFile symbol
 
    pkgsMods <- findSymbolInPackages symbol
    return . nub . map (GHC.moduleNameString . GHC.moduleName) $ fileMods ++ pkgsMods
 
 
-findSymbolInFile :: SymbolName -> FilePath -> GHC.Ghc [GHC.Module]
-findSymbolInFile symbol file = do
-   loadFile
-   filterM (containsSymbol symbol) =<< fileModules
-   where
-   loadFile = do
-      let noPhase = Nothing
-      target <- GHC.guessTarget file noPhase
-      GHC.setTargets [target]
-      let handler err = GHC.printException err >> return GHC.Failed
-      _ <- GHC.handleSourceError handler (GHC.load GHC.LoadAllTargets)
-      return ()
-
-   fileModules = map GHC.ms_mod <$> GHC.getModuleGraph
+findSymbolInFile :: SymbolName -> GHC.Ghc [GHC.Module]
+findSymbolInFile symbol =
+   filterM (containsSymbol symbol) =<< map GHC.ms_mod <$> GHC.getModuleGraph
 
 
 findSymbolInPackages :: SymbolName -> GHC.Ghc [GHC.Module]
